@@ -24,12 +24,26 @@ setMethod("show", "Pca", function(object) myPcaPrint(object))
 setMethod("summary", "Pca", function(object, ...){
     vars <- getEigenvalues(object)
     vars <- vars/sum(vars)
-    importance <- rbind("Standard deviation" = getSdev(object),
-                        "Proportion of Variance" = round(vars,5),
-                        "Cumulative Proportion" = round(cumsum(vars), 5))
+
+    ## If k < p, use the stored initial eigenvalues and total explained variance, if any
+    if(length(vars) < object@rank)
+    {
+        if(length(object@eig0) > 0 && length(object@totvar0) > 0)
+        {
+            vars <- object$eig0/object$totvar0
+            vars <- vars[1:object$k]
+        } else
+            vars <- NULL
+    }
+    importance <- if(is.null(vars)) rbind("Standard deviation" = getSdev(object))
+                  else              rbind("Standard deviation" = getSdev(object),
+                                         "Proportion of Variance" = round(vars,5),
+                                         "Cumulative Proportion" = round(cumsum(vars), 5))
+
     colnames(importance) <- colnames(getLoadings(object))
     new("SummaryPca", pcaobj=object, importance=importance)
 })
+
 setMethod("show", "SummaryPca", function(object){
 
     cat("\nCall:\n")
@@ -39,6 +53,12 @@ setMethod("show", "SummaryPca", function(object){
 
     cat("Importance of components:\n")
     print(object@importance, digits = digits)
+
+    if(nrow(object@importance) == 1)
+        cat("\nNOTE: Proportion of Variance and Cumulative Proportion are not shown",
+            "\nbecause the chosen number of components k =", object@pcaobj@k,
+            "\nis smaller than the rank of the data matrix =", object@pcaobj@rank, "\n")
+
     invisible(object)
 })
 
@@ -48,7 +68,7 @@ setMethod("predict",   "Pca", function(object, ...){
     predict(getPrcomp(object), ...)
 })
 setMethod("screeplot", "Pca", function(x, ...){
-    screeplot(getPrcomp(x), ...)
+    pca.screeplot(x, ...)
 })
 
 setMethod("biplot",    "Pca", function(x, choices=1L:2L, scale=1, ...){
@@ -135,19 +155,29 @@ myPcaPrint <- function(x, print.x=FALSE, print.loadings=FALSE, ...) {
 ## Internal function to calculate the score and orthogonal distances and the
 ##  appropriate cutoff values for identifying outlying observations
 ##
-##  obj  - the Pca object
-##  data -
+##  obj  - the Pca object. From this object will be used:
+##          - k, eigenvalues, scores
+##          - center and scale
+##  data - the original data (not centered and scaled)
 ##  r    - rank
 ##  crit - criterion for computing cutoff for SD and OD
 ##
-##  - cutoff for score distances: sqrt(qchisq(crit, k)
+##  - cutoff for score distances: sqrt(qchisq(crit, k))
 ##  - cutoff for orthogonal distances: Box (1954)
 ##
+
 pca.distances <- function(obj, data, r, crit=0.975) {
     .distances(data, r, obj, crit)
 }
+
 .distances <- function(data, r, obj, crit=0.975) {
 
+##  VT::28.07.2020 -  - add adjusted for skewed data mode. The 'skew'
+##      parameter will be used only in PcaHubert() to control how to
+##      calculate the distances and their cutoffs.
+
+    skew <- inherits(obj,"PcaHubert") && obj$skew
+
     ## remember the criterion, could be changed by the user
     obj@crit.pca.distances <- crit
 
@@ -167,6 +197,15 @@ pca.distances <- function(obj, data, r, crit=0.975) {
     obj@sd <- sqrt(mahalanobis(as.matrix(obj@scores[,1:nk]), rep(0, nk), diag(obj@eigenvalues[1:nk], ncol=nk)))
     obj@cutoff.sd <- sqrt(qchisq(crit, obj@k))
 
+    if(skew)
+    {
+        ## In this case (only valid when called from PcaHubert) the
+        ##  SD are the observations' adjusted outlyingness, and the corresponding
+        ##  cutoff value is derived in the same way as for the orthogonal distances.
+        obj@sd <- obj@ao
+        obj@cutoff.sd <- .crit.od(obj@sd, crit=crit, method="skewed")
+    }
+
     ## Compute the orthogonal distances and the corresponding cutoff value
     ##  For each point this is the norm of the difference between the
     ##  centered data and the back-transformed scores
@@ -185,7 +224,9 @@ pca.distances <- function(obj, data, r, crit=0.975) {
     ##  the rank of the data matrix - otherwise set it to 0
     obj@cutoff.od <- 0
     if(obj@k != r) {
-        obj@cutoff.od <- .crit.od(obj@od, crit=crit, classic=inherits(obj,"PcaClassic"))
+        ## the method used for computing the cutoff depends on (a) classic/robust and (b) skew
+        mx <- if(inherits(obj,"PcaClassic")) "classic" else if(skew) "skewed" else "medmad"
+        obj@cutoff.od <- .crit.od(obj@od, crit=crit, method=mx)
     }
 
     ## flag the observations with 1/0 if the distances are less or equal the
@@ -197,24 +238,37 @@ pca.distances <- function(obj, data, r, crit=0.975) {
     return(obj)
 }
 
-.crit.od <- function(od, crit=0.975, umcd=FALSE, quan, classic=FALSE)
+## Adjusted for skewness cutoff of the orthogonal distances:
+##  - cutoff = the largest od_i smaller than Q3({od}) + 1.5 * exp(3 * medcouple({od})) * IQR({od})
+##
+.crit.od <- function(od, crit=0.975, method=c("medmad", "classic", "umcd", "skewed"), quan)
 {
-    od <- od^(2/3)
-    if(classic)
+    method <- match.arg(method)
+    if(method == "skewed")
     {
-        t <- mean(od)
-        s <- sd(od)
-    }else if(umcd)
+        mc <- robustbase::mc(od, maxit=1000)
+        e3mc <- if(mc < 0) 1 else exp(3*mc)
+        cx <- quantile(od, 0.75) + 1.5 * e3mc * IQR(od)
+        cv <- max(od[which(od < cx)])   # take the largest od, smaller than cx
+    } else
     {
-        ms <- unimcd(od, quan=quan)
-        t <- ms$tmcd
-        s <- ms$smcd
-    }else
-    {
-        t <- median(od)
-        s <- mad(od)
+        od <- od^(2/3)
+        if(method == "classic")
+        {
+            t <- mean(od)
+            s <- sd(od)
+        }else if(method == "umcd")
+        {
+            ms <- unimcd(od, quan=quan)
+            t <- ms$tmcd
+            s <- ms$smcd
+        }else
+        {
+            t <- median(od)
+            s <- mad(od)
+        }
+        cv <- (t + s * qnorm(crit))^(3/2)
     }
-    cv <- (t + s * qnorm(crit))^(3/2)
     cv
 }
 
@@ -230,7 +284,7 @@ pca.distances <- function(obj, data, r, crit=0.975) {
 
 ## This is from MM in robustbase, but I want to change it and
 ##  therefore took a copy. Later will update in 'robustbase'
-##  I want to use not 'scale()', but doScale to which i can pass also
+##  I want to use not 'scale()', but doScale to which I can pass also
 ##   a function.
 .classPC <- function(x, scale=FALSE, center=TRUE,
 		    signflip=TRUE, via.svd = n > p, scores=FALSE)
@@ -360,9 +414,32 @@ kernelEVD <- function(x, scale=FALSE, signflip=TRUE){
     }
 }
 
+pca.screeplot <- function (obj, k, type = c("barplot", "lines"), main = deparse1(substitute(obj)), ...)
+{
+    type <- match.arg(type)
+    pcs <- if(is.null(obj@eig0) || length(obj@eig0) == 0) obj@eigenvalues else obj@eig0
+    k <- if(missing(k)) min(10, length(pcs))
+         else           min(k, length(pcs))
+
+    xp <- seq_len(k)
+
+    dev.hold()
+    on.exit(dev.flush())
+    if (type == "barplot")
+        barplot(pcs[xp], names.arg = names(pcs[xp]), main = main,
+            ylab = "Variances", ...)
+    else {
+        plot(xp, pcs[xp], type = "b", axes = FALSE, main = main,
+            xlab = "", ylab = "Variances", ...)
+        axis(2)
+        axis(1, at = xp, labels = names(pcs[xp]))
+    }
+    invisible()
+}
+
 ## Score plot of the Pca object 'obj' - scatterplot of ith against jth score
 ##  with superimposed tollerance (0.975) ellipse
-pca.scoreplot <- function(obj, i=1, j=2, main, id.n=0, ...)
+pca.scoreplot <- function(obj, i=1, j=2, main, id.n, ...)
 {
     if(missing(main))
     {
@@ -370,13 +447,14 @@ pca.scoreplot <- function(obj, i=1, j=2, main, id.n=0, ...)
     }
 
     x <- cbind(getScores(obj)[,i], getScores(obj)[,j])
+    rownames(x) <- rownames(getScores(obj))
 
 ## VT::11.06.2012
 ##  Here we assumed that the scores are not correlated and
 ##  used a diagonal matrix with the eigenvalues on the diagonal to draw the ellipse
 ##  This is not the case with PP methods, therefore we compute the covariance of
 ##  the scores, considering only the non-outliers
-##      (based on sore and orthogonal distances)
+##      (based on score and orthogonal distances)
 ##
 ##    ev <- c(getEigenvalues(obj)[i], getEigenvalues(obj)[j])
 ##    cpc <- list(center=c(0,0), cov=diag(ev), n.obs=obj@n.obs)
